// -*-C++-*-
//
// File:      DalListView.cc
// Author:    Jorgo Bakker (jbakker@astro.estec.esa.nl)
// Generated: Tue Jul 25 14:56:37 MET DST 2000

#include "DalListView.h"
#include "DalListViewItem.h"
#include "DalListViewFolder.h"

#include <qpopupmenu.h>
#include <qtooltip.h>
#include <qmessagebox.h>



DalListView::DalListView(Folder* root,QWidget* parent, const char* name) :
  QListView(parent,name),
  _type(DalListViewable::Any),
  _root(root)
{
  // setup signal forwarding:
  connect(this,SIGNAL(selectionChanged(QListViewItem*)),
	  this,SLOT(handleSelectionChanged(QListViewItem*)));

  connect(this,SIGNAL(rightButtonPressed(QListViewItem*,const QPoint&,int)),
	  this,SLOT(handleRightClicked(QListViewItem*,const QPoint&,int)));

  connect(this,SIGNAL(doubleClicked(QListViewItem*)),
	  this,SLOT(handleDoubleClicked(QListViewItem*)));
  connect(this,SIGNAL(returnPressed(QListViewItem*)),
	  this,SLOT(handleDoubleClicked(QListViewItem*)));

  if (_root) {
    addColumn("");
    setColumnWidth(0,200);
    //setRootIsDecorated(false);
    setRootIsDecorated(true);
 
    DalListViewItem* current=0;
    current=new DalListViewFolder(_root,this);
  }
  else {
    QToolTip::add(this,
		  /*"Left-click on item to select\n"*/
		  "Double-click on item to open\n"
		  "Right-click on item to get menu");
    addColumn("Name");
    addColumn("Type");
    // setSorting(1,false);
    // setMultiSelection(true);
  }

}

DalListView::~DalListView()
{
  clear();
}

QString DalListView::selectedQualifiedName()
{
  DalListViewItem* d=selectedItem();
  if (d) return d->qualifiedName();
  else return QString();
}

DalListViewItem* DalListView::openQualifiedName(const QString& n)
{
  /* browse all siblings to find the best matching root item
   */

  DalListViewItem* d=firstChild();
  DalListViewItem* best=0;
  int idxBest=-1;

  while (d) {
    QString cur=d->qualifiedName();
    if (cur == n) return d;

    int idxCur=n.find(cur);
    if ( idxCur != -1) { // it is a match: but better than last?
      if (idxCur > idxBest) {
	idxBest=idxCur;
	best=d;
      }
    }

    d=d->nextSibling();
  }

  if (best) return best->open(n);
  return (DalListViewItem*)0;
}


/*
  Different behavior depending on Tree/Flat view...
 */
void DalListView::handleLeftClicked (DalListViewItem * d)
{

  if (!d) return; // always!

  if (!_root) { // Flat view:
    // Hidden way to have DalListViewAnyFile figuring out whether
    // it is a Dataset...
    d->setOpen(false); 


    // flat view: can select?
    if (_type && d->type()) {
      if ( (_type|d->type()) == _type) emit addSelection(d);
    }

    return;
  }
  
  DalListViewItem* parent=currentItem();
  if (!parent) return; //something very wrong?

  parent->setOpen(true);
  
  DalListViewItem* current=parent->firstChild();
  while ( current !=0 ) {
    if (current->text(0) == d->text(0)) {
      current->setOpen(true);
      setCurrentItem(current);
      setSelected(current,true);
      ensureItemVisible(current);
      return;
    }
    current = current->nextSibling();
  }

  if (d->type() == DalListViewItem::DataSetType) {
    parent->insertItem(d);
    handleLeftClicked(d);
  }

}

/*
  Flat only! Open up the selection and show its contents
 */
void DalListView::handleDoubleClicked(DalListViewItem* i)
{
  if (!i) return;
  if (_root) return;
  
  i->openChildren(this);

  /* TODO:
  // figure out and ask whether we want to parse the possible datasets?


  bool hasUnqualifiables=false;
  DalListViewItem* current=(DalListViewItem*)firstChild();
  while (current && !hasUnqualifiables) {
    hasUnqualifiables= (current->type() == DalListViewable::None);
    current = current->nextSibling();
  }
  if (!hasUnqualifiables) return;

  bool parseOnEntry=(QMessageBox("Unknown files",
				 "Found files that may be datasets\n"
				 "Do you want to identify them now?",
				 QMessageBox::Information,
				 QMessageBox::Yes|QMessageBox::Default,
				 QMessageBox::No,
				 QMessageBox::Cancel|QMessageBox::Escape).exec()
		     == QMessageBox::Yes);
  if (!parseOnEntry) return;
  current=(DalListViewItem*)firstChild();
  while (current) {
    current->setOpen(false);
    current = current->nextSibling();
  }  
  */
}

void DalListView::gotoParent()
{
  if (!_root) return;
  
  DalListViewItem* item=selectedItem();
  if (!item) return;

  item = item->parent();
  if (!item) return;

  ensureItemVisible(item);
  setSelected(item,true);
}

void DalListView::jumpTo(DalListViewItem* item)
{
  if (!item)  return;
  if (!_root) return;
  ensureItemVisible(item);
  setSelected(item,true);  
}


/* ----------------------------------------------------------------------------
  converters/forwarders
 */

DalListViewItem* DalListView::currentItem()
{
  return convert(QListView::currentItem());
}

DalListViewItem* DalListView::selectedItem()
{
  return convert(QListView::selectedItem());
}

DalListViewItem* DalListView::firstChild()
{
  return convert(QListView::firstChild());
}


/* ----------------------------------------------------------------------------
  Private slots: only used for forwarding real items
    Unfortunately, we !have! to cast the items ;-(

    But be safe, do dynamic_cast only!!!
 */

void DalListView::handleSelectionChanged(QListViewItem* i)
{
  if (DalListViewItem* d=convert(i)) {
    emit leftClicked(d);
  }
}

void DalListView::handleDoubleClicked(QListViewItem* i)
{
  if (DalListViewItem* d=convert(i))
    emit doubleClicked(d);
}

void DalListView::handleRightClicked(QListViewItem* i,const QPoint& atPos,int)
{

  DalListViewItem* d=convert(i);
  if (!d) return;
  setSelected(d,true);

  if (_root) return; // popup only allowed in flat pane


  QPopupMenu menu(this);
  menu.insertItem("&Open",1);
  menu.insertItem("&Select",2);

  /* NOT IMPLEMENTED
     menu.insertSeparator();
     menu.insertItem("&Cut",3);
     menu.insertItem("&Copy",4);
     menu.insertItem("&Paste",5);
     menu.insertSeparator();
     menu.insertItem("&Rename",6);
  */
  
  // enable/disable items in menu
  menu.setItemEnabled( 2,_type && d->type() && ((_type|d->type()) == _type) );
  /*
  if (_type != Any) {
    if (d->type() != _type)  menu.setItemEnabled(2,false);
  }
  */

  /* NOT IMPLEMENTED
     menu.setItemEnabled(3,false);
     menu.setItemEnabled(4,false);
     menu.setItemEnabled(5,false);
     menu.setItemEnabled(6,false);
  */

  //
  menu.setMouseTracking(true);  
  switch (menu.exec(atPos)) {
  case 1:
    emit doubleClicked(d);
    break;
  case 2:
    emit addSelection(d);
    break;
  }    
}

DalListViewItem* DalListView::convert(QListViewItem* i)
{
  return dynamic_cast<DalListViewItem*>(i);
}
