/*****************************************************************************
 *
 *  ParamDialog.cc                              XMM Science Analysis Subsystem
 *
 *  Parameter Dialog for the SAS GUI
 *
 *  (c) 1998  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: ParamDialog.cc,v 1.18 2001/06/07 10:28:05 jbakker Exp $
 *
 *****************************************************************************/


#include "ParamDialog.h"        // Application headers
#include "ParamWidget.h"
#include "LYTReader.h"
#include "GuiBuilder.h"
#include "LayoutPrinter.h"
#include <SasFileLocator.h>

#include <cassert>             // C headers

#include <iostream>           // C++ headers
#include <strstream>
#include <fstream>
#include <algo.h>
#include <map>

#include <qdialog.h>            // Qt headers
#include <qpushbutton.h>
#include <qtooltip.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qframe.h>
#include <qgroupbox.h>
#include <qlayout.h>
#include <qstring.h>

#include <qapplication.h>

//================================================================
//  Class ParamDialog
//================================================================

//----------------------------
//  Constructors & destructors
//----------------------------

/*** Constructor ***/

ParamDialog::ParamDialog(ParameterInfo* info,
			 QWidget *parent,
			 const char *name,
			 bool useTabs,             // Use tabs if no layout file
			 const char* okButton,     // Label on 'Ok' button
			 const char* lytFileName)  // Name of layout file
 : QDialog(parent, name, true), _canvas(0)
{
    /*** Preconditions ***/

    assert(info);

    
    /*** Copy the ParameterInfo so we can revert ***/
        
    _originalInfo = info;
    _info = new ParameterInfo(*info);

    /*** Create the push buttons ***/

    QPushButton* runButton = new QPushButton(okButton, this );
    connect(runButton, SIGNAL(clicked()), SLOT(run()) );
    QToolTip::add(runButton,"Run the task");

    QPushButton* defaultButton = new QPushButton("Defaults", this);
    connect(defaultButton, SIGNAL(clicked()), SLOT(revertToDefault()) );
    QToolTip::add(defaultButton,"Revert to default values");

    QPushButton* cancelButton = new QPushButton("Cancel", this);
    connect(cancelButton, SIGNAL(clicked()), SLOT(reject()) );
    QToolTip::add(cancelButton,"Close the dialog");

    /*
     *  Create a canvas with a scrollable view. The canvas is embedded
     *  in a frame to provide a border which scrolls with the canvas.
     */
    QVBoxLayout* vbox = new QVBoxLayout(this);
    



    /*** Add parameter widgets to the canvas ***/
    // Find out the longest parameter name (pixels), and pass it on
    QSize labelSize(0,0);
    {
      list<string> names=_info->names();
      for ( list<string>::const_iterator it=names.begin();
	    it!=names.end();it++) {
	//	QSize s(QLabel(QString((*it).c_str()),0).sizeHint());
	QString qstr((*it).c_str());
	QSize s(QLabel(qstr,0).sizeHint());
	if (s.width() > labelSize.width()) labelSize.setWidth(s.width());
	if (s.height() > labelSize.height()) labelSize.setHeight(s.height());
      }
    }

    // 
    _layout = createLayout(useTabs, lytFileName);
    GuiBuilder* builder = new GuiBuilder(this,labelSize);
    _layout->build(builder);
    _canvas=builder->widget();

    vbox->addWidget(_canvas);
    vbox->addStretch(1);
    vbox->addSpacing(5);

    /*** Print the layout with a LayoutPrinter ***/

    if(0)   // Enable for debugging
    {
        cout << endl << "LayoutPrinter:" << endl;
	LayoutBuilder* printer = new LayoutPrinter(&cout);
	_layout->build(printer);
    }

    /*** Geometry management ***/
    QHBoxLayout* hbox=new QHBoxLayout(vbox);
    hbox->addStretch(1);
    hbox->addWidget(runButton);
    hbox->addSpacing(2);
    hbox->addWidget(defaultButton);
    hbox->addSpacing(2);
    hbox->addWidget(cancelButton);
    hbox->addSpacing(5);
    vbox->addSpacing(5);


    QSize ds=sizeHint();
    if (ds.width() > maxWidth) ds.setWidth(maxWidth);
    

    resize(ds);

    
    /*** Use the task name as the dialog title ***/

    setCaption(info->taskName().c_str());
}


/*** Destructor ***/

ParamDialog::~ParamDialog()
{
    delete _canvas;     // Delete view ...
    delete _info;       // ... then model
    delete _layout;
}


//-----------------------------
//  Layout
//-----------------------------

/*** Create a layout description for the parameter widgets [ PRIVATE ] ***/

Layout* ParamDialog::createLayout(bool useTabs, const char* lytFileName)
{
    /*
     *  NOTE: maxPerTab should be related to maximum dialog height,
     *  so that tabbed layouts don't need scroll bars.
     */

    const int maxPerTab = 10;

    Layout* layout = 0;

    SasFileLocator locator("");
    string filename(_info->taskName() + ".lyt");
    if(lytFileName)
        filename = string(lytFileName) + ".lyt";
    string lytFile = locator.locate("config/" + filename, "", false);

    // Use the layout file if it exists

    LYTReader lytReader(_info);
    if(lytFile.length() && lytReader.open(lytFile))
    {
	lytReader.parse(layout);
	if(!layout)
	    layout = new ColumnLayout;
    }
    else if(useTabs && int(_info->names().size()) > maxPerTab)
        layout = createTabbedLayout(maxPerTab);
    else
        layout = createColumnLayout();

    return layout;
}


/*** Construct simple column layout for '_info' [ PRIVATE ] ***/

Layout* ParamDialog::createColumnLayout()
{
    ColumnLayout* column = new ColumnLayout;
    list<string> namelist = _info->names();
    list<string>::iterator it = namelist.begin();
    while(it != namelist.end())
    {
	string name = *it++;
	Parameter* p = (*_info)[name];
	if(p)
	{
	    SimpleLayout* pl = new SimpleLayout(p);
	    column->addItem(pl);
	}
    }
    return column;
}


/*** Construct tabbed layout for '_info' [ PRIVATE ] ***/

Layout* ParamDialog::createTabbedLayout(int maxPerTab)
{
    PagedLayout* tabpad  = new PagedLayout;
    ColumnLayout* column = new ColumnLayout;

    list<string> namelist = _info->names();
    list<string>::iterator it = namelist.begin();
    int item = 1;
    int page = 0;
    while(it != namelist.end())
    {
        string name = *it++;
	Parameter* p = (*_info)[name];
	if(p)
        {
	    SimpleLayout* pl = new SimpleLayout(p);
	    column->addItem(pl);
	    
	    if(item++ == maxPerTab || it == namelist.end())
	    {
	        ostrstream ostr;
		ostr << page++ << ends;
		column->setName(ostr.str());
		tabpad->addItem(column);
		column = new ColumnLayout;
		item = 1;
	    }
	}
    }
    return tabpad;
}


//-----------------------------
//  Button actions
//-----------------------------

/*** Run the task [ PRIVATE SLOT ] ***/

void ParamDialog::run()
{
    *_originalInfo = *_info;
    accept();
}


/*** Revert to default values  [ PRIVATE SLOT ] ***/

void ParamDialog::revertToDefault()
{
    _info->revertToDefault();
}

/** function that does the trick **/
extern "C" bool popUpDialog(ParameterInfo* info)
{
  QApplication* app=0;
  if (!qApp) {
      char** argv=0;
      int    argc=0;
      app=new QApplication(argc,argv);
  }
  bool useTabs=true;
  ParamDialog* dialog = new ParamDialog(info, 0, 0, useTabs, "Run");
  
  bool ok = (dialog->exec() != 0);
  delete dialog;

  /* Cannot delete app, as qApp is global available. Deleting it ruins
     any Qt application that follows
  */
  //delete app;

  return ok;
}
