/*****************************************************************************
 *
 *  Process.cc                                  XMM Science Analysis Subsystem
 *
 *  Class for UNIX process
 *
 *  (c) 1999 ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: Process.cc,v 1.4 2001/05/10 13:03:52 jbakker Exp $
 *
 *****************************************************************************/

#include "Process.h"            // Application headers

#include <iostream>           // C++ headers
#include <vector>
#include <string>

#include <sys/wait.h>           // C headers
#include <csignal>
#include <unistd.h>


//================================================================
//  Class Process
//================================================================

/*
 *  Constructor
 *
 *  The constructor creates a Process object to control the system
 *  process. However, the process is not actually started until
 *  "execute()" is called. This allows recovery from errors
 *  (e.g. missing file), without using exceptions.
 */

Process::Process(const string& name, QObject* parent)
  : QObject(parent,0)
{
    _name         = name;
    _status       = NOTSTARTED;
    _exitCode     = 0;
    _childPid     = 0;
    _outNotifier  = 0;
    _errNotifier  = 0;
    _outfd        = 0;
    _errfd        = 0;
    _openPipes    = 0;
    _outConnected = false;
    _errConnected = false;
    _inRequired   = false;
}


/*
 *  Destructor
 *
 *  Kill the process if the Process object is deleted.
 */

Process::~Process()
{
    kill();
}


/*
 *  Execute a program as a separate system process with specified arguments
 *
 *  The Process object can be reused to run another process, by calling
 *  "execute()" again, after the process has finished. However, it is
 *  recommended that a new Process object is created each time. The
 *  "execute" method is only distinct from the constructor to simplify
 *  handling.
 */

void Process::execute(char** argv)
{
    if(_status == RUNNING)            // Already running!
        return;

    _status    = RUNNING;
    _exitCode  = 0;
    _openPipes = 0;
    _childPid  = 0;


    /*** Create pipe to provide stdin ***/

    int fdin[2];   // parent writes to 1, child reads from 0
    if(_inRequired)
    {
        if(pipe(fdin) < 0)
	{
	    cerr << "pipe error" << endl;
	    ::exit(1);
	}
    
	_infd = fdin[1];
    }

    /*** Create pipe to catch stdout from Process ***/

    int fdout[2];  // child writes to 1, parent reads from 0
    if(_outConnected)
    {
	if(pipe(fdout) < 0)
	{
	    cerr << "pipe error" << endl;
	    ::exit(1);
	}
	_outfd = fdout[0];
	_outNotifier = new QSocketNotifier(fdout[0], QSocketNotifier::Read, this);
	connect(_outNotifier, SIGNAL(activated(int)), SLOT(outReceived(int)) );
	connect(&_outBuffer, SIGNAL(line(const char*)),
		SIGNAL(outputMessage(const char*)));
	_openPipes++;
    }

    int fderr[2];  // child writes to 1, parent reads from 0
    if(_errConnected)
    {
	if(pipe(fderr) < 0)
	{
	   cerr << "pipe error" << endl;
	   ::exit(1);
	}
    
	_errfd = fderr[0];
	_errNotifier = new QSocketNotifier(fderr[0], QSocketNotifier::Read, this);
	connect(_errNotifier, SIGNAL(activated(int)), SLOT(errReceived(int)) );
	connect(&_errBuffer, SIGNAL(line(const char*)),
		SIGNAL(errorMessage(const char*)));
	_openPipes++;
    }


    /*** Fork child process ***/

    pid_t pid;

    if ((pid = fork()) < 0) 
    {
        cerr << "fork failed" << endl;
	::exit(1);
    
    } else if (pid == 0) {            // Child

        /*** Redirect input ***/

        if(_inRequired)
        {
  	    ::close(fdin[1]);
	    dup2(fdin[0],STDIN_FILENO);
	    ::close(fdin[0]);
	}

        /*** Redirect output ***/

        if(_outConnected)
	{
	    ::close(fdout[0]);
	    dup2(fdout[1],STDOUT_FILENO);
	    ::close(fdout[1]);
	}

	if(_errConnected)
	{
	    ::close(fderr[0]);
	    dup2(fderr[1],STDERR_FILENO);
	    ::close(fderr[1]);
	}

        /*** Execute the Process ***/

	execvp(_name.c_str(),argv);
	::exit(127);                  // If 'exec' fails

    } else {                          // Parent

        _childPid = pid;
	::close(fdin[0]);
	if(_outConnected)
	    ::close(fdout[1]);
	if(_errConnected)
	    ::close(fderr[1]);
    }
}


/*** Private slots to handle data received from child ***/

void Process::outReceived(int fd)
{
    dataReceived(fd, _outNotifier, _outBuffer);
}

void Process::errReceived(int fd)
{
    dataReceived(fd, _errNotifier, _errBuffer);
}


/*
 *  Allow the child process to receive standard input from application.
 *  If this is not enabled, standard input will be read from the terminal 
 *  from which the application was started.
 */

void Process::enableInput(bool enable)
{
    _inRequired = enable;
}


/*** Send input to the task ***/

void Process::sendInput(const char* text)
{
    if(_inRequired)
    {
        // No check is made that the pipe can accept all the input.
        // 
        // A better approach would be to use a QSocketNotifier and
        // a FIFO configured for O_NONBLOCK. sendInput() would write to 
        // the FIFO and enable the notifier. When the notifier fired it 
        // would move data from the FIFO to the pipe and then disable the 
        // notifier (unless the write would have caused the application 
        // to block.

        write(_infd, text, strlen(text));
    }
}


/*
 *  Receive data from the process and line-buffer it, so that output from
 *  different processes can be interleaved more cleanly.
 *
 *  Task termination is detected by reaching EOF on all the pipes. This
 *  is much cleaner than catching UNIX signals (which is difficult to
 *  do in an object-oriented way, unless we register with a central
 *  signal handler).
 *
 *  Note: At least one output signal must be connected, otherwise the
 *        the Process object does not receive notification when the process
 *        terminates, and cannot emit a "done" signal.
 */

void Process::dataReceived(int fd, QSocketNotifier* notifier,
			   LineBuffer& buffer)
{
    notifier->setEnabled(false);

    /*
     *  Allocate a buffer to receive the message. If the message 
     *  is larger than the buffer, the notifier simply fires again.
     *  However, a larger (up to 10240 for Solaris) improves performance.
     *  The following figures show relative timings under Solaris:
     *
     *   MAXLINE  seconds
     *      1      16.7
     *     50      16.4
     *    100      11.4
     *    512       4.7
     *   2048       3.4
     *  20000       3.1
     */

    const int MAX_LINE = 10240;
    char  lineBuf[MAX_LINE];

    int n = read(fd, lineBuf, MAX_LINE);
    if(n > 0)
    {
        lineBuf[n] = 0;
	buffer.append(lineBuf);
    } else {

        // End-of-file reached

	disconnect(notifier);
	delete notifier;
	notifier = 0;
	::close(fd);

	buffer.flush();

	if(--_openPipes == 0)       // Check that all outputs are finished
	{
	    waitFor();              // Wait for process to exit
	    emit done(_exitCode);
	}
    }
    if(notifier)
        notifier->setEnabled(true);
}


/*
 *  Wait for process to complete, then return exit code. We need to
 *  do this, so we don't get a zombie process.
 *
 *  Note: This method causes the caller's event loop to block. It could
 *  also deadlock if the output pipe became full, since the blocked event 
 *  loop prevents data being read from the pipe.
 *
 *  Applications should normally use the "done" signal to receive asynchonous
 *  notification of process completion, instead of calling "waitFor".
 */

int Process::waitFor()
{
    if(_status == DONE || _status == FAILED)
        return 0;

    int status;
    while (waitpid(_childPid, &status, 0) == 0) 
      ;
  
    _status   = WIFEXITED(status) ? DONE : FAILED;

    if( WIFEXITED(status) )
        _exitCode = WEXITSTATUS(status);
    else
        _exitCode = -1;   // Failed or killed

    return _exitCode;
}


/*
 *  Test status of child process
 *
 *  This should not normally be needed, since the process will
 *  be running from the call to execute() until receipt of the
 *  "done()" signal.
 */

Process::Status Process::childStatus()
{
    // The status is cached, because waitpid reports an abnormal exit,
    // if is is called a second time, after a normal exit.

    if(_status == RUNNING)
    {
        int status;
	bool exists = _childPid && (waitpid(_childPid, &status, WNOHANG) == 0);

        if(!exists)
	    _status = WIFEXITED(status) ? DONE : FAILED;
    }
    return _status;
}


/*** Test whether the child process is running ***/

bool Process::childExists()
{
    return _childPid && (waitpid(_childPid, NULL, WNOHANG) == 0);
}


/*** Kill the process and wait for it to exit ***/

void Process::kill()
{
    if(childExists())
    {
        ::kill(_childPid, SIGKILL);
	waitFor();
    }
}


/*** Receives notification when a Qt signal is connected to an output ***/

void Process::connectNotify(const char* signal)
{
    if(string(signal) == "2outputMessage(const char*)")
        _outConnected = true;
    else if(string(signal) == "2errorMessage(const char*)")
        _errConnected = true;
}
