/*****************************************************************************
 *
 *  ValidatedEdit.cc                            XMM Science Analysis Subsystem
 *
 *  Qt widget for editing constrained strings
 *
 *  (c) 1999  ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: ValidatedEdit.cc,v 1.5 2001/05/10 13:03:53 jbakker Exp $
 *
 *****************************************************************************/

#include <iostream>            // C++ headers

#include <qlineedit.h>           // Qt headers
#include <qwidget.h>
#include <qkeycode.h>
#include <qtimer.h>
#include <qcolor.h>
#include <qpalette.h>

#include "ValidatedEdit.h"       // Application headers


//================================================================
//  Abstract class ValidatedEdit
//================================================================

/*** Constructor ***/

ValidatedEdit::ValidatedEdit(QWidget *parent, const char *name)
  : QLineEdit(parent, name)
{
    _revertDelay    = 2000;
    _beepEnabled    = true;
    _lastGoodCursor = 0;
    _lastCursor     = 0;
    _lastText       = "";

    _timer = new QTimer(this);
    connect(_timer, SIGNAL(timeout()),SLOT(revertAndBeep()) );

    connect(this, SIGNAL(textChanged(const QString&)), SLOT(newText())); 
}


/*** Handle key press event ***/

void ValidatedEdit::keyPressEvent(QKeyEvent *e)
{
    // Remember cursor position before QLineEdit gets key

    _lastCursor = cursorPosition();
    if(isInRange())
        _lastGoodCursor = _lastCursor;

    // Normalise value on Return/Enter, otherwise let QLineEdit handle it.

    if(!(e->state() & ControlButton))
    {
	switch(e->key())
	{
	case Key_Enter:
	case Key_Return:
	    setTextToValue();
	    e->accept();
	    break;

	default:
	    QLineEdit::keyPressEvent(e);
	    break;
	}
    }
}


/*** Handle loss of focus ***/

void ValidatedEdit::focusOutEvent(QFocusEvent* e)
{
    setTextToValue();
    QLineEdit::focusOutEvent(e);
}


/***  Invoked when text in underlying QLineEdit widget changes ***/

void ValidatedEdit::newText()
{
    validateText();
}


/*** Revert to last valid; invoked by timeout ***/

void ValidatedEdit::revertAndBeep()
{
    setTextToValue();
    setCursorPosition(_lastGoodCursor);
    beep();
}


/*** Set the colour of the text ***/

void ValidatedEdit::setTextColor(const QColor& color)
{
    QPalette p(palette());
    QColorGroup n(p.normal());
    QColorGroup colgrp(n.foreground() , n.background(), 
		       n.light(), n.dark(), n.mid(),
		       color, n.base() );

    setPalette(QPalette(colgrp,p.disabled(),colgrp));
}


/*** Enable warning sound ***/

void ValidatedEdit::setEnableBeep(bool enable)
{
    _beepEnabled = enable;
}


/*** Make warning sound ***/

void ValidatedEdit::beep()
{
    if(_beepEnabled)
        cerr << '\a' << flush;
}


/*
 *  Set the delay used before reverting to last good value, 
 *  when an invalid entry is made:
 *
 *  t<0 : Disable - continue to show invalid entry in red
 *  t=0 : Revert immediately - This may make it difficult
 *        to edit with certain min/max ranges.
 *  t>0 : Show invalid entry in red, then revert to last
 *        good value after delay
 */

void ValidatedEdit::setRevertDelay(int msec)
{
    _revertDelay = msec;

    //  Start the timer if current value is out of range 
    //  and the timer is not already running.

    if(!_timer->isActive() && !isInRange())
        markValidity(false);
}


/*
 *  Set colour highlighting to reflect validity.
 *  If invalid, start timer to revert after a short delay.
 */

void ValidatedEdit::markValidity(bool valid)
{
    if(valid)
    {
        setTextColor(black);
	if(_timer->isActive())
	    _timer->stop();
    } else {

        setTextColor(red);
	beep();
    
	if(_revertDelay >= 0)
	    _timer->start(_revertDelay,true);
    }
}


/*
 *  Check validity of current text.
 *  Revert if syntactically invalid; highlight if out-of-range.
 */

void ValidatedEdit::validateText()
{
    if(!isValid())                        // Syntax error
    {
        int cursor = _lastCursor;
        setText(_lastText);               // Undo last key
	setCursorPosition(cursor);
    }
    else
    {
        bool inRange = isInRange();
	markValidity(inRange);
	if(inRange)
	    setValueToText();             // Accept if in range

	_lastText   = text();             // Save for undo
	_lastCursor = cursorPosition();
    }
}
