/*****************************************************************************
 *
 *  ValidatedEdit.h                             XMM Science Analysis Subsystem
 *
 *  Qt widget for editing constrained strings
 *
 *  (c) 1999 ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: ValidatedEdit.h,v 1.3 2001/05/10 13:03:53 jbakker Exp $
 *
 *****************************************************************************/

#ifndef VALIDATEDEDIT_H
#define VALIDATEDEDIT_H

#include <qlineedit.h>              // Qt headers
#include <qtimer.h>
#include <qstring.h>

#include <iostream>


//================================================================
//  Abstract class ValidatedEdit
//
//  ValidatedEdit is an abstract class which may be subclassed to provide
//  widgets for entering strings which are constrained in certain ways
//  (e.g. numbers, dates).
//
//  The subclass must implement the following virtual methods:
//
//     setTextToValue()   Set text to representation of _value
//     setValueToText()   Parse text() and set _value if it has changed
//     bool isValid()     Test hard constraint (e.g. syntax valid)
//     bool isInRange()   Test soft constraint (e.g. value in range)
//
//  Soft constraints may be temporarily violated, since it may be
//  necessary to go from one valid value to another, via an intermediate
//  invalid state. The widget beeps (if enabled) and turns red whilst
//  it is invalid. If the value is not corrected, the widget automatically
//  reverts to the last valid value, after a short delay.
//
//  Hard constraints are typically syntactic ones and can often be
//  implemented as a regular expression, using QRegExp.
//
//  The widget may provide methods to set/get its value and to set/get
//  the soft constraint (e.g. allowed range of number). The widget may
//  emit a valueChanged() signal in setValueToText(), but only if the
//  value has changed. The value is only updated when the text is valid;
//  hence a getValue() method will only ever return valid values.
//
//  Notes:
//  - isInRange() tests the text, rather than the current value, since 
//  the value is only updated if it is in range.
//
//================================================================

class ValidatedEdit : public QLineEdit
{
    Q_OBJECT

public:

    // Constructors and destructors
    ValidatedEdit(QWidget *parent=0, const char *name=0);

    // Configuration
    void setRevertDelay(int msec);
    void setEnableBeep(bool enable = true);

    // Enable to check for redundant updates
#if( 0 )
    void repaint(bool t) {
        cout << "repaint " << name() << endl;
	QLineEdit::repaint(t); 
    }
    void repaint(int x, int y, int w, int h, bool erase=TRUE) {
        cout << "repaint " << name() << endl;
        QLineEdit::repaint(x,y,w,h,erase);
    }
    void repaint(const QRect &rect, bool erase=TRUE) {
        cout << "repaint " << name() << endl;
        QLineEdit::repaint(rect,erase);
    }
    void paintEvent(QPaintEvent * e) {
        cout << "paintEvent " << name() << endl;
	QLineEdit::paintEvent(e);
    }
    void update() {
        cout << "update " << name() << endl;
	QLineEdit::update();
    }
    void update(int x, int y, int w, int h) {
        cout << "update " << name() << endl;
	QLineEdit::update(x,y,w,h);
    }
    void update(const QRect &r) {
        cout << "update " << name() << endl;
	QLineEdit::update(r);
    }
    void setText(const char* t) {
        cout << "setText " << name() << endl;
	QLineEdit::setText(t);
    }
#endif

protected:

    // Event handlers
    void keyPressEvent(QKeyEvent *);
    void focusOutEvent(QFocusEvent*);

    // Validation
    void validateText();

    // Implemented by sub-class
    virtual void setTextToValue() = 0;
    virtual void setValueToText() = 0;
    virtual bool isValid() { return true; }
    virtual bool isInRange() { return true; }

private slots:

    void newText();
    void revertAndBeep();
    
private:

    // Disabled copy constructor and operator=
    ValidatedEdit(const ValidatedEdit&);
    ValidatedEdit& operator=(const ValidatedEdit&) {return *this;}

    // Warnings
    void markValidity(bool valid);
    void setTextColor(const QColor& color);
    void beep();

    // Member data
    int      _revertDelay;
    QTimer*  _timer;
    bool     _beepEnabled;
    QString  _lastText;
    int      _lastCursor;
    int      _lastGoodCursor;
};

#endif
