/*****************************************************************************
 *
 *  TaskManager.cc                              XMM Science Analysis Subsystem
 *
 *  Controller to manage execution of tasks
 *
 *  (c) 1999 ESA  -  Written by Jon Brumfitt, Aurora
 *
 *  $Id: TaskManager.cc,v 1.8.2.2 2002/11/18 10:49:02 jbakker Exp $
 *
 *****************************************************************************/

#include <iostream>                // C++ headers
#include <strstream>

#include "TaskManager.h"          // Application headers
#include "ParamInfoSet.h"
#include "ParamDialog.h"
#include "AlertDialog.h"

#include <qmessagebox.h>             // Qt headers


//================================================================
//  Class TaskManager
//================================================================

/*** Constructor ***/

TaskManager::TaskManager(QWidget* parent)
  : QObject(parent,0), _parent(parent)
{
    _preferences = 0;
}


/*** Destructor ***/

TaskManager::~TaskManager()
{
}


/*** Open parameter dialog and run task [ PRIVATE SLOT ] ***/

void TaskManager::editParam(const char* taskName)
{
    assert(taskName);

    ParameterInfo* info = 
        ParamInfoSet::instance()->getParamInfo(string(taskName));

    if(!info)
        return;               // Parameters not found


    /*** Get user preferences which control dialog layout ***/

    bool useTabs = true;
    if(_preferences)
    {
        Parameter* parTabs = _preferences->getParameter("useTabs");
	if(parTabs)
	    useTabs = parTabs->boolValue();
    }


    /*** Display the parameter dialog, then run the task ***/

    ParamDialog* dialog = new ParamDialog(info, _parent, 0, useTabs, "Run");
    if(dialog->exec())
    { 
        if(!_tasks.empty())
	{
	    string message = "The following tasks are already running.\n"
	                     "You may queue the task for later execution,\n"
	                     "if it conflicts with these tasks.\n";
	    
	    list<SasTask*>::const_iterator it = _tasks.begin();
	    while(it != _tasks.end())
	    {
	        SasTask* t = *it++;
		message += string("\n  ") + t->name();
	    }
	    
	    /*
	     *  Tasks are already running. Ask the user whether to run now,
	     *  cancel or queue until other tasks have finished. The SasTask
	     *  object makes a copy of the parameter info, so that parameters
	     *  may be edited once a task is queued or running.
	     */
	    
	    switch(QMessageBox::information(
				        _parent,
					taskName,           // Caption
					message.c_str(),    // Message
				        "Run now",          // Button 0
				        "Cancel",           // Button 1
				        "Queue",            // Button 2
				        0,                  // Default button
				        1 ))                // Escape button
	      {
	          case 0: {	                            // Run now
		      SasTask* task = new SasTask(info, this, _preferences);
		      runTask(task);
		      break;
		  }
	          case 1: {                                 // Cancel
		      break;
		  }
	          case 2: {                                 // Run later
		      SasTask* task = new SasTask(info, this, _preferences);
		      _queue.push_back(task);
		      break;
		  }
	      }
	} else {
	  delete dialog;
	    SasTask* task = new SasTask(info, this, _preferences);
	    runTask(task);
	}
    } else {                   // Parameter dialog canceled
        delete dialog;
    }
}


/*** Run the task with specified parameters [ PRIVATE ] ***/

void TaskManager::runTask(SasTask* task)
{
    // Reset "don't show again" warnings
    AlertDialog::reset();
    
    // Run the task
    _tasks.push_back(task);
    connect(task,SIGNAL(done(int,SasTask*)),SLOT(taskDone(int,SasTask*)));

    showInfo(string("Task '") + task->name() + "' started");
    emit enableKill(true);

    task->execute();
}


/*** Kill the most recently started task ***/

void TaskManager::killLastTask()
{
    if(!_tasks.empty()) {
        SasTask* task = _tasks.back();
	task->kill();
        _tasks.pop_back();
    }
}


/*** Reply whether any tasks are running ***/

bool TaskManager::tasksRunning()
{
    return(!_tasks.empty());
}


/*** Set the user preferences ***/

void TaskManager::setPreferences(GuiPreferences* preferences)
{
    _preferences = preferences;
}


/*** Receives notification when a task has finsihed [ PRIVATE SLOT ] ***/

void TaskManager::taskDone(int exitCode, SasTask* task)
{
    if(exitCode == -1)
        showInfo(string("Task '") + task->name() + "' killed");
    else
        showInfo(string("Task '") + task->name() + "' finished");

    _tasks.remove(task);
    emit enableKill(!_tasks.empty());

    delete task;

    runQueuedTask();
}


/*
 * Start a queued task if one is available and nothing is running
 * [ PRIVATE ]
 */

void TaskManager::runQueuedTask()
{
    if(_tasks.empty() && !_queue.empty())
    {
        SasTask* task1 = _queue.front();
	_queue.pop_front();

        runTask(task1);
    }
}


/*** Display a transient message [ PRIVATE ] ***/

void TaskManager::showInfo(const string& text)
{
    emit transientMessage(text.c_str());
}


